
import 'package:flutter/material.dart';

class CustomClipperImage extends CustomClipper<Path> {
  final bool positive;
  final double dy;
  const CustomClipperImage({this.positive=true,this.dy=0});
  static const double slop=36;
  static const double nonslop=96;

  @override
  Path getClip(Size size) {
    var path = Path();

    double process(bool x,double os)=>positive?x?os:os+dy:x?size.width-os:size.height-os+dy;

      path.moveTo(process(true,0), process(false,slop));

      path.quadraticBezierTo(process(true,4), process(false,2),
          process(true,6), process(false,0));
      path.lineTo(process(true,nonslop), process(false,0));
      path.quadraticBezierTo(process(true,nonslop+2), process(false,2),
          process(true,size.width), process(false,slop));
      path.lineTo(process(true,size.width), process(false,size.height - 27));
      path.quadraticBezierTo(process(true,size.width-3), process(false,size.height-3),
          process(true,size.width-27), process(false,size.height));
      path.lineTo(process(true,27), process(false,size.height));
      path.quadraticBezierTo(process(true,3), process(false,size.height-3),
          process(true,0), process(false,size.height-27));
      path.lineTo(process(true,0), process(false,12));
      path.close();

    return path;
  }


  @override
  bool shouldReclip(CustomClipper oldClipper) {
    return true;
  }
}

class ChatClipper extends CustomClipper<Path> {
  final bool left;
  final bool last;
  const ChatClipper({this.left=true,this.last=false});
  static const double hrzPad=tail+6;
  static const double rad=9;
  static const double radArc=0.96;
  static const double tail=15;

  @override
  Path getClip(Size size) {
    var path = Path();

    path.moveTo(hrzPad, rad);

    path.quadraticBezierTo(hrzPad+rad*(1-radArc), rad*(1-radArc), hrzPad+rad, 0);
    path.lineTo(size.width-hrzPad-rad, 0);
    path.quadraticBezierTo(size.width-hrzPad-rad*(1-radArc), rad*(1-radArc),
        size.width-hrzPad, rad);
    path.lineTo(size.width-hrzPad, size.height - rad);
    if(!last){
      path.quadraticBezierTo(size.width-hrzPad-rad*(1-radArc), size.height-rad*(1-radArc),
          size.width-hrzPad-rad, size.height);
      path.lineTo(hrzPad+rad, size.height);
      path.quadraticBezierTo(hrzPad+rad*(1-radArc), size.height-rad*(1-radArc),
          hrzPad, size.height-rad);
    }
    else if(!left){
      path.quadraticBezierTo(size.width-hrzPad+rad*(1-radArc), size.height-rad*(1-radArc),
          size.width-hrzPad+tail, size.height);
      path.lineTo(hrzPad+rad, size.height);
      path.quadraticBezierTo(hrzPad+rad*(1-radArc), size.height-rad*(1-radArc),
          hrzPad, size.height-rad);
    }
    else{
      path.quadraticBezierTo(size.width-hrzPad-rad*(1-radArc), size.height-rad*(1-radArc),
        size.width-hrzPad-rad, size.height);
      path.lineTo(hrzPad-tail, size.height);
      path.quadraticBezierTo(hrzPad-rad*(1-radArc), size.height-rad*(1-radArc),
          hrzPad, size.height-rad);
    }
    path.lineTo(hrzPad, rad);
    path.close();

    return path;
  }


  @override
  bool shouldReclip(CustomClipper oldClipper) {
    return true;
  }
}


class ClipShadowShadowPainter extends CustomPainter {
  final Shadow shadow;
  final CustomClipper<Path> clipper;
  const ClipShadowShadowPainter({required this.shadow, required this.clipper});
  @override
  void paint(Canvas canvas, Size size) {
    var paint = shadow.toPaint();
    var clipPath = clipper.getClip(size).shift(shadow.offset);
    canvas.drawPath(clipPath, paint);
  }
  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    return true;
  }
}

@immutable
class ClipShadowPath extends StatelessWidget {
  final Shadow shadow;
  final CustomClipper<Path> clipper;
  final Widget child;
  ClipShadowPath({
    required this.shadow,
    required this.clipper,
    required this.child,
  });
  @override
  Widget build(BuildContext context) {
    return CustomPaint(
        painter: ClipShadowShadowPainter(
            clipper: this.clipper, shadow: this.shadow),
        child: ClipPath(child: child, clipper: this.clipper));
  }
}