import 'dart:math';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:pc_plugins/utils/utils.dart';

class numberFmt implements TextInputFormatter{
  final bool frac;
  numberFmt({this.frac=false});

  @override
  TextEditingValue formatEditUpdate(TextEditingValue oldValue, TextEditingValue newValue) {
    if(newValue.text.isEmpty)return newValue;
    if(frac) {
      return double.tryParse(newValue.text)!=null?newValue:oldValue;
    }else{
      return int.tryParse(newValue.text)!=null?newValue:oldValue;
    }
  }
}

class dollarFmt implements TextInputFormatter{
  dollarFmt();

  @override
  TextEditingValue formatEditUpdate(TextEditingValue oldValue, TextEditingValue newValue) {
    ///if choose all and delete all text
    if(newValue.text.isEmpty)return newValue.copyWith(text: "0.00");

    final n=formateDollarStr(newValue.text,addSign: false);
    ///if new text is illegal
    if(n==null) {
      return oldValue;
    }
    else{
      ///if delete, or .
      if(newValue.text.length+1==oldValue.text.length&&
          newValue.text.replaceAll(",", "").replaceAll(".", "")==
              oldValue.text.replaceAll(",", "").replaceAll(".", "")){
        for(int i=0;i<oldValue.text.length;i++){
          if(oldValue.text[i]!=newValue.text[i]){
            if(oldValue.text[i]=="."){
              String newx=oldValue.text.replaceRange(i-1, i, "");
              if(newx.startsWith("."))newx="0"+newx;
              return newValue.copyWith(text: formateDollarStr(newx, addSign: false));
            }
            else return newValue.copyWith(text: n);
          }
        }
      }
      // final n1=countComma(newValue.text);
      return newValue.copyWith(text: n);
    }
  }

  int countComma(String s){
    int r=0;
    for(int i=0;i<s.length;i++){
      if(s[i]==",")r++;
    }
    return r;
  }
}

/// Format incoming numeric text to fit the format of (###) ###-#### ##...
class UsNumberTextInputFormatter extends TextInputFormatter {
  @override
  TextEditingValue formatEditUpdate(
      TextEditingValue oldValue,
      TextEditingValue newValue
      ) {
    final int newTextLength = newValue.text.length;
    int selectionIndex = newValue.selection.end;
    int usedSubstringIndex = 0;
    final StringBuffer newText = StringBuffer();
    if(newValue.text.contains('-')) {
      return TextEditingValue(
        text: newValue.text,
        selection: TextSelection.collapsed(offset: selectionIndex),
      );
    }
//      newText.write('(');
//      if (newValue.selection.end >= 1)
//        selectionIndex++;
//    }
//    if (newTextLength >= 4) {
//      newText.write(newValue.text.substring(0, usedSubstringIndex = 3) + ') ');
//      if (newValue.selection.end >= 3)
//        selectionIndex += 2;
//    }
    if (newTextLength >= 3) {
      newText.write(newValue.text.substring(0, usedSubstringIndex = 2) + '-');
      if (newValue.selection.end >= 2)
        selectionIndex++;
    }
//    if (newTextLength >= 11) {
//      newText.write(newValue.text.substring(6, usedSubstringIndex = 10) + ' ');
//      if (newValue.selection.end >= 10)
//        selectionIndex++;
//    }
    // Dump the rest.
    if (newTextLength >= usedSubstringIndex)
      newText.write(newValue.text.substring(usedSubstringIndex));
    return TextEditingValue(
      text: newText.toString(),
      selection: TextSelection.collapsed(offset: selectionIndex),
    );
  }
}

class NumberTextInputFormatter extends TextInputFormatter {
  @override
  TextEditingValue formatEditUpdate(
      TextEditingValue oldValue, TextEditingValue newValue) {
    String phoneNumber=newValue.text;
    int s=newValue.selection.end;

    if(phoneNumber.startsWith('1')||phoneNumber.startsWith('+')){               //added fix for +1 phone numbers
      s++;
      if(phoneNumber.length>=10&&oldValue.text.length<10)s++;
      if(phoneNumber.length>=5&&oldValue.text.length<5)s+=2;
    }
    else{
      if(phoneNumber.length>=8&&oldValue.text.length<8)s++;
      if(phoneNumber.length>=3&&oldValue.text.length<3)s+=2;
    }
    // if(phoneNumber.length>9&&oldValue.text.length<9)s++;
    if (phoneNumber.contains("\(") && !phoneNumber.contains("\)")) {
      // Remove the digit the user wanted to remove but couldn't b/c a paren
      // was in the way.
      phoneNumber = phoneNumber.substring(0, phoneNumber.length - 1);
    }

    // Only store the actual digits
    phoneNumber = phoneNumber.replaceAll(RegExp("[^0-9]"), "");

    // Don't let the user enter more digits than is possible
    int maxLength = phoneNumber.startsWith("1") ? 11 : 10;
    if (phoneNumber.length > maxLength) {
      phoneNumber = phoneNumber.substring(0, maxLength);
    }

    return newValue.copyWith(text: getFormattedPhoneNumber(phoneNumber),
    selection: TextSelection.collapsed(offset: s));
  }

  String getFormattedPhoneNumber(String _phoneNumber) {
    if (_phoneNumber.isEmpty) {
      return "";
    }

    String phoneNumber = _phoneNumber;
    bool addPlus = phoneNumber.startsWith("1");
    if (addPlus) phoneNumber = phoneNumber.substring(1);
    bool addParents = phoneNumber.length >= 3;
    bool addDash = phoneNumber.length >= 6;

    // +1
    String updatedNumber = "";
    if (addPlus) updatedNumber += "+1";

    // (222)
    if (addParents) {
      updatedNumber += "(";
      updatedNumber += phoneNumber.substring(0, 3);
      updatedNumber += ")";
    } else {
      updatedNumber += phoneNumber.substring(0);
      return updatedNumber;
    }

    // 111
    if (addDash) {
      updatedNumber += phoneNumber.substring(3, 6);
      updatedNumber += "-";
    } else {
      updatedNumber += phoneNumber.substring(3);
      return updatedNumber;
    }

    // 3333
    updatedNumber += phoneNumber.substring(6);
    return updatedNumber;
  }

}

class heightFormatter extends TextInputFormatter{
  static const defaultText="0'0\"";
  @override
  TextEditingValue formatEditUpdate(TextEditingValue oldValue, TextEditingValue newValue) {
    List<String> raw=newValue.text.replaceAll("\"", "").split("'");
    if(raw.length!=2||(raw[0].isNotEmpty&&int.tryParse(raw[0])==null)
        ||(raw[1].isNotEmpty&&int.tryParse(raw[1])==null)){
      if(int.tryParse(newValue.text)!=null){
        return newValue.copyWith(text: "${int.tryParse(newValue.text)}'0\"");
      }
      if(getLegalHeight(oldValue.text)!=null){
        return oldValue.copyWith(text: fomate(getLegalHeight(oldValue.text)!));
      }
      return TextEditingValue(text: fomate(getLegalHeight(defaultText)!));
    }
    if(raw[0].isEmpty){
      raw[0]='0';
      return newValue.copyWith(text: "${raw[0]}'${raw[1]}\"",
          selection: TextSelection.collapsed(offset: newValue.selection.end+1));
    }if(raw[1].isEmpty){
      raw[1]='0';
      return newValue.copyWith(text: "${raw[0]}'${raw[1]}\"",
          selection: TextSelection.collapsed(offset: newValue.selection.end+1));
    }
    int i=int.tryParse(raw[0])!;
    int ii=int.tryParse(raw[1])!;
    final String ich='$i'.substring(1)+(ii==0?'':'$ii');
    ii=int.tryParse(ich.isEmpty?'0':ich)!;
    final selct=newValue.selection.end+(i>9?1:0);
    return newValue.copyWith(text: "${'$i'[0]}'${ii}\"",
        selection: TextSelection.collapsed(offset:
        min("${'$i'[0]}'${ii}\"".length-1, selct)));
  }

  String fomate(List<dynamic> r){
    return "${r[0]}'${r[1]}\"";
  }
}

List<dynamic>? getLegalHeight(String? r){
  if(r==null||!r.endsWith("\""))return null;
  List<String>l=r.substring(0,r.length-1).split("'");
  if(l.length!=2)return null;
  int? ft=int.tryParse(l[0].isEmpty?'0':l[0]);
  double? inch=double.tryParse(l[1].isEmpty?'0':l[1]);
  if(ft==null||inch==null)return null;
  return[ft,inch];
}
double? string2inch(String s){
  List? l=getLegalHeight(s);
  if(l==null)return null;
  return l[0]*12+l[1];
}
String? valiLegalHeight(String? r)=>getLegalHeight(r)==null?"please enter valid Height":null;
final _mobileFormatter = NumberTextInputFormatter();
